@@ -26,18 +26,20 @@
 #include <signal.h>
 #include <sys/ioctl.h>
 
-#include "../libsysfs/dlist.h"
-#include "../libsysfs/sysfs/libsysfs.h"
-#include "../libdevmapper/libdevmapper.h"
-#include "../libcheckers/checkers.h"
+#include <parser.h>
+#include <vector.h>
+#include <memory.h>
+#include <sysfs/dlist.h>
+#include <sysfs/libsysfs.h>
+#include <libdevmapper.h>
+#include <checkers.h>
+#include <path_state.h>
+
 #include "main.h"
 #include "devinfo.h"
 #include "config.h"
 #include "pgpolicies.h"
-#include "parser.h"
 #include "dict.h"
-#include "vector.h"
-#include "memory.h"
 #include "callout.h"
 #include "debug.h"
 
@@ -169,12 +171,12 @@ devinfo (struct path *curpath)
 	curpath->claimed = get_claimed(curpath->dev_t);
 
 	/*
-	 * get path state
+	 * get path state, no message collection, no context
 	 */
 	select_checkfn(curpath);
-	curpath->state = checkpath(curpath->sg_dev_t, curpath->checkfn);
-dbg("state : %i", curpath->state);
-dbg("sg_dev_t : %s", curpath->sg_dev_t);
+	curpath->state = checkpath(curpath->sg_dev_t, curpath->checkfn,
+				   NULL, NULL);
+	dbg("path %s state : %i", curpath->dev, curpath->state);
 	
 	/*
 	 * get path prio
@@ -453,10 +455,20 @@ print_path (struct path * pp, int style)
 
 	printf ("%s ", pp->dev);
 
-	if (pp->state)
+	switch (pp->state) {
+	case PATH_UP:
 		printf ("[ready ] ");
-	else
+		break;
+	case PATH_DOWN:
 		printf ("[faulty] ");
+		break;
+	case PATH_SHAKY:
+		printf ("[shaky] ");
+		break;
+	default:
+		dbg("undefined path state");
+		break;
+	}
 
 	if (pp->claimed)
 		printf ("[claimed] ");
@@ -740,18 +752,10 @@ dm_reinstate(char * mapname, char * path)
         return r;
 }
 
-/*
- * Transforms the path group vector into a proper device map string
- */
-void
-assemble_map (struct multipath * mp)
+static int
+select_selector (struct multipath * mp)
 {
-	int i, j;
-	int shift, freechar;
-	char * p;
-	vector pgpaths;
-	char * selector;
-	int selector_args;
+	int i;
 	struct mpentry * mpe;
 	struct hwentry * hwe;
 	struct path * pp;
@@ -766,8 +770,8 @@ assemble_map (struct multipath * mp)
 	 */
 
 	/* 1) set internal default */
-	selector = conf->default_selector;
-	selector_args = conf->default_selector_args;
+	mp->selector = conf->default_selector;
+	mp->selector_args = conf->default_selector_args;
 
 	/* 2) override by controler wide settings */
 	for (i = 0; i < VECTOR_SIZE(conf->hwtable); i++) {
@@ -775,9 +779,9 @@ assemble_map (struct multipath * mp)
 
 		if (strcmp(hwe->vendor, pp->vendor_id) == 0 &&
 		    strcmp(hwe->product, pp->product_id) == 0) {
-			selector = (hwe->selector) ?
+			mp->selector = (hwe->selector) ?
 				   hwe->selector : conf->default_selector;
-			selector_args = (hwe->selector_args) ?
+			mp->selector_args = (hwe->selector_args) ?
 				   hwe->selector_args : conf->default_selector_args;
 		}
 	}
@@ -787,12 +791,26 @@ assemble_map (struct multipath * mp)
 		mpe = VECTOR_SLOT(conf->mptable, i);
 
 		if (strcmp(mpe->wwid, mp->wwid) == 0) {
-			selector = (mpe->selector) ?
+			mp->selector = (mpe->selector) ?
 				   mpe->selector : conf->default_selector;
-			selector_args = (mpe->selector_args) ?
+			mp->selector_args = (mpe->selector_args) ?
 				   mpe->selector_args : conf->default_selector_args;
 		}
 	}
+	return 0;
+}
+
+/*
+ * Transforms the path group vector into a proper device map string
+ */
+void
+assemble_map (struct multipath * mp)
+{
+	int i, j;
+	int shift, freechar;
+	char * p;
+	vector pgpaths;
+	struct path * pp;
 
 	p = mp->params;
 	freechar = sizeof(mp->params);
@@ -806,10 +824,10 @@ assemble_map (struct multipath * mp)
 	p += shift;
 	freechar -= shift;
 	
-	for (i = 0; i < VECTOR_SIZE(mp->pg); i++) {
+	for (i = VECTOR_SIZE(mp->pg) - 1; i >= 0; i--) {
 		pgpaths = VECTOR_SLOT(mp->pg, i);
-		shift = snprintf(p, freechar, " %s %i %i",
-			         selector, VECTOR_SIZE(pgpaths), selector_args);
+		shift = snprintf(p, freechar, " %s %i %i", mp->selector,
+				 VECTOR_SIZE(pgpaths), mp->selector_args);
 		if (shift >= freechar) {
 			fprintf(stderr, "mp->params too small\n");
 			exit(1);
@@ -818,8 +836,8 @@ assemble_map (struct multipath * mp)
 		freechar -= shift;
 
 		for (j = 0; j < VECTOR_SIZE(pgpaths); j++) {
-			shift = snprintf(p, freechar, " %s",
-					 (char *)VECTOR_SLOT(pgpaths, j));
+			pp = VECTOR_SLOT(pgpaths, j);
+			shift = snprintf(p, freechar, " %s", pp->dev_t);
 			if (shift >= freechar) {
 				fprintf(stderr, "mp->params too small\n");
 				exit(1);
@@ -836,10 +854,9 @@ assemble_map (struct multipath * mp)
 }
 
 static int
-setup_map (vector pathvec, vector mp, int slot)
+setup_map (vector pathvec, struct multipath * mpp)
 {
 	struct path * pp;
-	struct multipath * mpp;
 	int i, iopolicy;
 	char iopolicy_name[POLICY_NAME_SIZE];
 	int fd;
@@ -849,8 +866,6 @@ setup_map (vector pathvec, vector mp, int slot)
 	char curparams[PARAMS_SIZE];
 	int op;
 
-	mpp = VECTOR_SLOT(mp, slot);
-
 	/*
 	 * don't bother if devmap size is unknown
 	 */
@@ -935,9 +950,17 @@ setup_map (vector pathvec, vector mp, int slot)
 	}
 
 	/*
-	 * layered mpp->paths reordering :
+	 * select the appropriate path group selector and args
+	 */
+	select_selector(mpp);
+
+	/*
+	 * layered map computation :
 	 *  1) sort paths by priority
-	 *  2) apply selected grouping policy to paths
+	 *  2) separate failed paths in a tersary PG
+	 *  3) separate shaky paths in a secondary PG
+	 *  4) apply selected grouping policy to valid paths
+	 *  5) reorder path groups by summed priority
 	 */
 
 	/*
@@ -946,7 +969,13 @@ setup_map (vector pathvec, vector mp, int slot)
 	mpp->paths = sort_pathvec_by_prio(mpp->paths);
 
 	/*
-	 * 2) apply selected grouping policy
+	 * 2) & 3)
+	 */
+	group_by_status(mpp, PATH_DOWN);
+	group_by_status(mpp, PATH_SHAKY);
+
+	/*
+	 * 4) apply selected grouping policy
 	 */
 	if (iopolicy == MULTIBUS)
 		one_group (mpp);
@@ -955,7 +984,7 @@ setup_map (vector pathvec, vector mp, int slot)
 		one_path_per_group (mpp);
 
 	if (iopolicy == GROUP_BY_SERIAL)
-		group_by_serial (mpp, slot);
+		group_by_serial (mpp);
 
 	if (iopolicy == GROUP_BY_PRIO)
 		group_by_prio (mpp);
@@ -965,6 +994,11 @@ setup_map (vector pathvec, vector mp, int slot)
 		return 1;
 	}
 
+	/*
+	 * 5)
+	 */
+	sort_pg_by_summed_prio(mpp);
+
 	/*
 	 * transform the mp->pg vector of vectors of paths
 	 * into a mp->params strings to feed the device-mapper
@@ -1157,7 +1191,8 @@ usage (char * progname)
 		"\t   group_by_prio\t\t- 1 priority group per priority lvl\n" \
 		"\n" \
 		"\tdevice\t\tlimit scope to the device's multipath\n" \
-		"\t\t\t(udev-style $DEVNAME reference, eg /dev/sdb)\n" \
+		"\t\t\t(udev-style $DEVNAME reference, eg /dev/sdb\n" \
+		"\t\t\tor a device map name)\n" \
 		);
 
 	exit(1);
@@ -1201,6 +1236,7 @@ main (int argc, char *argv[])
 {
 	vector mp;
 	vector pathvec;
+	struct multipath * mpp;
 	int k;
 	int arg;
 	extern char *optarg;
@@ -1339,6 +1375,20 @@ main (int argc, char *argv[])
 
 	coalesce_paths(mp, pathvec);
 
+	/*
+	 * may be conf->dev is a mapname
+	 * if so, only reconfigure this map
+	 */
+	for (k = 0; k < VECTOR_SIZE(mp); k++) {
+		mpp = VECTOR_SLOT(mp, k);
+
+		if (0 == strncmp(mpp->alias, conf->dev, FILE_NAME_SIZE) ||
+		    0 == strncmp(mpp->wwid, conf->dev, FILE_NAME_SIZE)) {
+			setup_map(pathvec, mpp);
+			goto out;
+		}
+	}
+
 	if (conf->verbosity > 1) {
 		print_all_path(pathvec);
 		print_all_mp(mp);
@@ -1347,9 +1397,12 @@ main (int argc, char *argv[])
 	if (conf->verbosity > 1)
 		fprintf(stdout, "#\n# device maps :\n#\n");
 
-	for (k = 0; k < VECTOR_SIZE(mp); k++)
-		setup_map(pathvec, mp, k);
+	for (k = 0; k < VECTOR_SIZE(mp); k++) {
+		mpp = VECTOR_SLOT(mp, k);
+		setup_map(pathvec, mpp);
+	}
 
+out:
 	/*
 	 * signal multipathd that new devmaps may have come up
 	 */
