@@ -20,7 +20,7 @@
 #define _MAIN_H
 
 /* local includes */
-#include "vector.h"
+#include <vector.h>
 #include "dev_t.h"
 
 /* global defs */
@@ -29,6 +29,7 @@
 #define PATH_STR_SIZE   16
 #define PARAMS_SIZE	256
 #define FILE_NAME_SIZE	256
+#define DEV_T_SIZE	32
 #define DEF_TIMEOUT	60000
 #define DM_TARGET	"multipath"
 #define PIDFILE		"/var/run/multipathd.pid"
@@ -57,7 +58,6 @@ struct sg_id {
 	int channel;
 	int scsi_id;
 	int lun;
-	int scsi_type;
 	short h_cmd_per_lun;
 	short d_queue_depth;
 	int unused1;
@@ -72,8 +72,8 @@ struct scsi_dev {
 
 struct path {
 	char dev[FILE_NAME_SIZE];
-	char dev_t[FILE_NAME_SIZE];
-	char sg_dev_t[FILE_NAME_SIZE];
+	char dev_t[DEV_T_SIZE];
+	char sg_dev_t[DEV_T_SIZE];
 	struct scsi_idlun scsi_id;
 	struct sg_id sg_id;
 	char wwid[WWID_SIZE];
@@ -84,7 +84,7 @@ struct path {
 	int state;
 	unsigned int priority;
 	int claimed;
-	int (*checkfn) (char *);
+	int (*checkfn) (char *, char *, void *);
 };
 
 struct multipath {
@@ -95,6 +95,8 @@ struct multipath {
 	vector paths;
 	vector pg;
 	char params[PARAMS_SIZE];
+	char * selector;
+	int selector_args;
 };
 
 /* Build version */
