@@ -12,20 +12,34 @@
 #include <wait.h>
 #include <sched.h>
 #include <errno.h>
+#include <syslog.h>
 #include <sys/mount.h>
 #include <sys/mman.h>
-#include "libsysfs/sysfs/libsysfs.h"
-#include "libsysfs/dlist.h"
-#include "libcheckers/checkers.h"
+
+/*
+ * libsysfs
+ */
+#include <sysfs/libsysfs.h>
+#include <sysfs/dlist.h>
+
+/*
+ * libcheckers
+ */
+#include <checkers.h>
+#include <path_state.h>
+
+/*
+ * libmultipath
+ */
+#include <parser.h>
+#include <vector.h>
+#include <memory.h>
 
 #include "hwtable.h"
 #include "dict.h"
-#include "parser.h"
-#include "vector.h"
 #include "devinfo.h"
-#include "memory.h"
 #include "copy.h"
-#include "log.h"
+#include "safe.h"
 #include "callout.h"
 
 #define CHILD_SS 16384
@@ -35,13 +49,22 @@
 #define MAPNAMESIZE 64
 #define TARGETTYPESIZE 16
 #define PARAMSSIZE 2048
-#define DEV_T_SIZE 2048
+#define DEV_T_SIZE 32
 
 #define PIDFILE "/var/run/multipathd.pid"
 #define CONFIGFILE "/etc/multipath.conf"
 #define CALLOUT_DIR "/var/cache/multipathd"
 
+#ifndef LOGLEVEL
+#define LOGLEVEL 5
+#endif
+
 #define MATCH(x, y) strncmp(x, y, strlen(x)) == 0
+#define LOG_MSG(a,b) \
+	if (strlen(a)) { \
+		syslog(LOG_WARNING, "%s: %s", b, a); \
+		memset(a, 0, MAX_CHECKER_MSG_SIZE); \
+	}
 
 #ifdef CLONE_NEWNS
 #ifdef ia64
@@ -61,20 +84,21 @@ pthread_cond_t *event;
 /*
  * structs
  */
-struct path
-{
+struct path {
+	int major;
+	int minor;
 	char sg_dev_t[DEV_T_SIZE];
-	int (*checkfn) (char *);
+	int state;
+	int (*checkfn) (char *, char *, void *);
+	void * checker_context;
 };
 
-struct paths
-{
+struct paths {
 	pthread_mutex_t *lock;
 	vector pathvec;
 };
 
-struct event_thread
-{
+struct event_thread {
 	pthread_t *thread;
 	pthread_mutex_t *waiter_lock;
 	int event_nr;
@@ -111,20 +135,8 @@ strvec_free (vector vec)
 	vector_free(vec);
 }
 
-static void
-pathvec_free (vector vec)
-{
-	int i;
-	struct path * pp;
-
-	for (i =0; i < VECTOR_SIZE(vec); i++)
-		if ((pp = VECTOR_SLOT(vec, i)) != NULL)
-			FREE(pp);
-	vector_free(vec);
-}
-
 static int
-select_checkfn(struct path *path_p, char *devname)
+select_checkfn(struct path *pp, char *devname)
 {
 	char vendor[8];
 	char product[16];
@@ -136,12 +148,12 @@ select_checkfn(struct path *path_p, char *devname)
 	/*
 	 * default checkfn
 	 */
-	path_p->checkfn = &readsector0;
+	pp->checkfn = &readsector0;
 	
 	r = get_lun_strings(vendor, product, rev, devname);
 
 	if (r) {
-		LOG(1, "can not get device strings");
+		syslog(LOG_ERR, "can not get scsi strings");
 		return r;
 	}
 
@@ -151,13 +163,13 @@ select_checkfn(struct path *path_p, char *devname)
 		    MATCH(hwe->product, product) &&
 		    hwe->checker_index > 0) {
 			get_checker_name(checker_name, hwe->checker_index);
-			LOG (2, "set %s path checker for %s",
+			syslog(LOG_INFO, "set %s path checker for %s",
 			     checker_name, devname);
-			path_p->checkfn = get_checker_addr(hwe->checker_index);
+			pp->checkfn = get_checker_addr(hwe->checker_index);
 			return 0;
 		}
 	}
-	LOG (2, "set readsector0 path checker for %s", devname);
+	syslog(LOG_INFO, "set readsector0 path checker for %s", devname);
 	return 0;
 }
 
@@ -167,13 +179,13 @@ exit_daemon (int status)
 	if (status != 0)
 		fprintf(stderr, "bad exit status. see daemon.log\n");
 
-	LOG (2, "umount ramfs");
+	syslog(LOG_INFO, "umount ramfs");
 	umount(CALLOUT_DIR);
 
-	LOG (2, "unlink pidfile");
-	unlink (PIDFILE);
+	syslog(LOG_INFO, "unlink pidfile");
+	unlink(PIDFILE);
 
-	LOG (1, "--------shut down-------");
+	syslog(LOG_NOTICE, "--------shut down-------");
 	exit(status);
 }
 
@@ -206,7 +218,7 @@ get_devmaps (void)
 	}
 
 	if (!names->dev) {
-		LOG (1, "no devmap found");
+		syslog(LOG_WARNING, "no devmap found");
 		goto out;
 	}
 
@@ -216,7 +228,7 @@ get_devmaps (void)
 		 */
 		names = (void *) names + next;
 		nexttgt = NULL;
-		LOG (3, "devmap %s :", names->name);
+		syslog(LOG_DEBUG, "devmap %s :", names->name);
 
 		if (!(dmt1 = dm_task_create(DM_DEVICE_STATUS)))
 			goto out;
@@ -227,19 +239,18 @@ get_devmaps (void)
 		if (!dm_task_run(dmt1))
 			goto out1;
 
-		LOG (4, "DM_DEVICE_STATUS ioctl done");
 		do {
 			nexttgt = dm_get_next_target(dmt1, nexttgt,
 						   &start,
 						   &length,
 						   &target_type,
 						   &params);
-			LOG (3, "\\_ %lu %lu %s", (unsigned long) start,
+			syslog(LOG_DEBUG, "\\_ %lu %lu %s", (unsigned long) start,
 						  (unsigned long) length,
 						  target_type);
 
 			if (!target_type) {
-				LOG (2, "   unknown target type");
+				syslog(LOG_INFO, "   unknown target type");
 				goto out1;
 			}
 			
@@ -249,7 +260,7 @@ get_devmaps (void)
 				vector_alloc_slot(devmaps);
 				vector_set_slot(devmaps, devmap);
 			} else
-				LOG (3, "   skip non multipath target");
+				syslog(LOG_DEBUG, "   skip non multipath target");
 		} while (nexttgt);
 
 out1:
@@ -266,7 +277,8 @@ out:
 static int
 updatepaths (struct paths *failedpaths)
 {
-	struct path *path_p;
+	int i;
+	struct path *pp;
 	struct sysfs_directory * sdir;
 	struct sysfs_directory * devp;
 	char path[FILENAMESIZE];
@@ -275,7 +287,7 @@ updatepaths (struct paths *failedpaths)
 	char attr_buff[17];
 	
 	if (sysfs_get_mnt_path(sysfs_path, FILENAMESIZE)) {
-		LOG(2, "can not find sysfs mount point");
+		syslog(LOG_ERR, "can not find sysfs mount point");
 		return 1;
 	}
 
@@ -285,48 +297,81 @@ updatepaths (struct paths *failedpaths)
 	}
 	sdir = sysfs_open_directory(path);
 	sysfs_read_dir_subdirs(sdir);
-
 	pthread_mutex_lock(failedpaths->lock);
-	pathvec_free(failedpaths->pathvec);
-	failedpaths->pathvec = vector_alloc();
 
 	dlist_for_each_data(sdir->subdirs, devp, struct sysfs_directory) {
 		if (blacklist(devp->name)) {
-			LOG (3, "%s blacklisted", devp->name);
+			syslog(LOG_DEBUG, "%s blacklisted", devp->name);
 			continue;
 		}
-
 		memset(attr_buff, 0, sizeof (attr_buff));
 		memset(attr_path, 0, sizeof (attr_path));
 
 		if (safe_sprintf(attr_path, "%s/block/%s/device/generic/dev",
 			sysfs_path, devp->name)) {
 			fprintf(stderr, "updatepaths: attr_path too small\n");
-			return 1;
+			continue;
 		}
 		if (0 > sysfs_read_attribute_value(attr_path, attr_buff, 17)) {
-			LOG(3, "no such attribute : %s",
+			syslog(LOG_DEBUG, "no such attribute : %s",
 				attr_path);
 			continue;
 		}
+		/*
+		 * detect if path already exists in path vector
+		 * if so, keep the old one for for checker context and
+		 * state persistance
+		 */
+		for (i = 0; i < VECTOR_SIZE(failedpaths->pathvec); i++) {
+			pp = VECTOR_SLOT(failedpaths->pathvec, i);
 
-		path_p = MALLOC(sizeof (struct path));
-		if (safe_snprintf(path_p->sg_dev_t, DEV_T_SIZE, "%s",
-				  attr_buff)) {
-			fprintf(stderr, "sg_dev_t too small\n");
-			return 1;
+			if (0 == strncmp(pp->sg_dev_t, attr_buff,
+					strlen(pp->sg_dev_t)))
+				break;
 		}
-		if (!select_checkfn(path_p, devp->name) &&
-		    checkpath(path_p->sg_dev_t, path_p->checkfn)) {
-			LOG(2, "discard %s as valid path", path_p->sg_dev_t);
-			FREE(path_p);
+		if (i < VECTOR_SIZE(failedpaths->pathvec)) {
+			syslog(LOG_INFO, "path checker already active : %s",
+				pp->sg_dev_t);
 			continue;
 		}
 
-		vector_alloc_slot(failedpaths->pathvec);
-		vector_set_slot(failedpaths->pathvec, path_p);
+		/*
+		 * ok, really allocate a path
+		 */
+		pp = MALLOC(sizeof(struct path));
 
-		LOG(2, "%s added to failedpaths", path_p->sg_dev_t);
+		if (safe_snprintf(pp->sg_dev_t, DEV_T_SIZE, "%s",
+				  attr_buff)) {
+			fprintf(stderr, "sg_dev_t too small\n");
+			FREE(pp);
+			continue;
+		}
+		pp->checker_context = MALLOC(MAX_CHECKER_CONTEXT_SIZE);
+
+		if (select_checkfn(pp, devp->name)) {
+			FREE(pp->checker_context);
+			FREE(pp);
+			continue;
+		}
+		pp->state = checkpath(pp->sg_dev_t, pp->checkfn, NULL,
+					pp->checker_context);
+
+		if (safe_sprintf(attr_path, "%s/block/%s/dev",
+			sysfs_path, devp->name)) {
+			fprintf(stderr, "updatepaths: attr_path too small\n");
+			continue;
+		}
+		if (0 > sysfs_read_attribute_value(attr_path, attr_buff, 17)) {
+			syslog(LOG_DEBUG, "no such attribute : %s",
+				attr_path);
+			continue;
+		}
+		sscanf(attr_buff, "%i:%i", &pp->major, &pp->minor);
+		
+		vector_alloc_slot(failedpaths->pathvec);
+		vector_set_slot(failedpaths->pathvec, pp);
+
+		syslog(LOG_NOTICE, "path checker startup : %s", pp->sg_dev_t);
 	}
 	pthread_mutex_unlock(failedpaths->lock);
 	sysfs_close_directory(sdir);
@@ -354,7 +399,7 @@ geteventnr (char *name)
 	}
 
 	if (!info.exists) {
-		LOG(1, "Device %s does not exist", name);
+		syslog(LOG_ERR, "Device %s does not exist", name);
 		info.event_nr = 0;
 		goto out;
 	}
@@ -369,13 +414,15 @@ static void *
 waitevent (void * et)
 {
 	struct event_thread *waiter;
+	char buff[1];
+	char * bin;
 
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 	waiter = (struct event_thread *)et;
 	pthread_mutex_lock (waiter->waiter_lock);
 
 	waiter->event_nr = geteventnr (waiter->mapname);
-	LOG (3, "waiter->event_nr = %i", waiter->event_nr);
+	syslog(LOG_DEBUG, "waiter->event_nr = %i", waiter->event_nr);
 
 	struct dm_task *dmt;
 
@@ -392,7 +439,15 @@ waitevent (void * et)
 
 out:
 	dm_task_destroy(dmt);
-	LOG (1, "devmap event on %s", waiter->mapname);
+
+	bin = MALLOC(strlen(multipath) + WWID_SIZE + 1);
+	sprintf(bin, "%s %s", multipath, waiter->mapname);
+
+	syslog(LOG_DEBUG, "%s", bin);
+	syslog(LOG_NOTICE, "devmap event on %s", waiter->mapname);
+	syslog(LOG_INFO, "%s : reconfigure multipath map",
+			waiter->mapname);
+	execute_program(multipath, buff, 1);
 
 	/*
 	 * tell waiterloop we have an event
@@ -410,20 +465,6 @@ out:
 	return (NULL);
 }
 
-static void
-log_failedpaths_vec (vector pathvec)
-{
-	int i;
-	struct path * pp;
-
-	LOG(1, "paths to check :");
-
-	for (i = 0; i < VECTOR_SIZE(pathvec); i++) {
-		pp = VECTOR_SLOT(pathvec, i);
-		LOG(1, "%s", pp->sg_dev_t);
-	}
-}
-
 static void *
 waiterloop (void *ap)
 {
@@ -437,7 +478,7 @@ waiterloop (void *ap)
 	char buff[1];
 	int i, j;
 
-	LOG (1, "start DM events thread");
+	syslog(LOG_NOTICE, "start DM events thread");
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 
 	/*
@@ -448,22 +489,14 @@ waiterloop (void *ap)
 	pthread_attr_init(&attr);
 	pthread_attr_setstacksize(&attr, 32 * 1024);
 
-	while (1) {
-		/*
-		 * upon waiter events and initial startup, do a preliminary
-		 * multipath exec, no signal to avoid recursion.
-		 * don't run multipath if we are waked from SIGHUP
-		 * (oper exec / checkers / hotplug) because it already ran
-		 */
-		if (from_sighup)
-			from_sighup = 0;
-		else
-			execute_program(multipath, buff, 1);
+	syslog(LOG_NOTICE, "initial reconfigure multipath maps");
+	execute_program(multipath, buff, 1);
 
+	while (1) {
 		/*
 		 * update devmap list
 		 */
-		LOG (2, "refresh devmaps list");
+		syslog(LOG_INFO, "refresh devmaps list");
 		if (devmaps != NULL)
 			strvec_free(devmaps);
 
@@ -471,21 +504,20 @@ waiterloop (void *ap)
 			/*
 			 * we're not allowed to fail here
 			 */
-			LOG (1, "can't get devmaps ... retry later");
+			syslog(LOG_ERR, "can't get devmaps ... retry later");
 			sleep(5);
 		}
 
 		/*
-		 * update failed paths list
+		 * update paths list
 		 */
-		LOG(2, "refresh failpaths list");
+		syslog(LOG_INFO, "refresh failpaths list");
 		updatepaths(failedpaths);
-		log_failedpaths_vec(failedpaths->pathvec);
 
 		/*
 		 * start waiters on all devmaps
 		 */
-		LOG (2, "start up event loops");
+		syslog(LOG_INFO, "start up event loops");
 
 		for (i = 0; i < VECTOR_SIZE(devmaps); i++) {
 			devmap = VECTOR_SLOT (devmaps, i);
@@ -530,7 +562,7 @@ waiterloop (void *ap)
 				pthread_mutex_unlock (wp->waiter_lock);
 			}
 			
-			LOG (1, "event checker startup : %s", wp->mapname);
+			syslog(LOG_NOTICE, "event checker startup : %s", wp->mapname);
 			pthread_create (wp->thread, &attr, waitevent, wp);
 			pthread_detach (*wp->thread);
 		}
@@ -549,36 +581,63 @@ static void *
 checkerloop (void *ap)
 {
 	struct paths *failedpaths;
-	struct path *path_p;
+	struct path *pp;
 	int i;
+	int newstate;
+	char buff[1];
+	char * bin;
+	char checker_msg[MAX_CHECKER_MSG_SIZE];
+
+	memset(checker_msg, 0, MAX_CHECKER_MSG_SIZE);
 
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 	failedpaths = (struct paths *)ap;
 
-	LOG (1, "path checkers start up");
+	syslog(LOG_NOTICE, "path checkers start up");
 
 	while (1) {
 		pthread_mutex_lock(failedpaths->lock);
-		LOG(3, "checking paths");
+		syslog(LOG_DEBUG, "checking paths");
 
 		for (i = 0; i < VECTOR_SIZE(failedpaths->pathvec); i++) {
-			path_p = VECTOR_SLOT(failedpaths->pathvec, i);
+			pp = VECTOR_SLOT(failedpaths->pathvec, i);
+			newstate = checkpath(pp->sg_dev_t, pp->checkfn,
+				      checker_msg, pp->checker_context);
 			
-			if (checkpath(path_p->sg_dev_t, path_p->checkfn)) {
+			if (newstate != pp->state) {
+				pp->state = newstate;
+				LOG_MSG(checker_msg, pp->sg_dev_t);
+
+				/*
+				 * don't trigger map reconfiguration for
+				 * path going down. It will be handled
+				 * in due time by DM event waiters
+				 */
+				if (newstate & (PATH_DOWN | PATH_SHAKY))
+					continue;
+
+				/*
+				 * reconfigure map now
+				 */
+				bin = MALLOC(strlen(multipath) + DEV_T_SIZE);
+				sprintf(bin, "%s -D %i %i", multipath,
+						pp->major, pp->minor);
+
+				syslog(LOG_DEBUG, "%s", bin);
+				syslog(LOG_INFO, "reconfigure multipath map");
+				execute_program(bin, buff, 1);
+
 				/*
 				 * tell waiterloop we have an event
 				 */
-				LOG(1, "path checker event on device %s",
-					path_p->sg_dev_t);
 				pthread_mutex_lock (event_lock);
 				pthread_cond_signal(event);
 				pthread_mutex_unlock (event_lock);
-			} else
-				LOG (2, "check path %s => down",
-				     path_p->sg_dev_t);
+			}
+			pp->state = newstate;
 		}
 		pthread_mutex_unlock(failedpaths->lock);
-		sleep(checkint ? checkint : CHECKINT);
+		sleep(checkint);
 	}
 
 	return (NULL);
@@ -684,17 +743,17 @@ prepare_namespace(void)
 	size_t size = 10;
 	struct stat statbuf;
 	
-	buf = MALLOC (sizeof (struct stat));
+	buf = MALLOC(sizeof(struct stat));
 
 	/*
 	 * create a temp mount point for ramfs
 	 */
 	if (stat (CALLOUT_DIR, buf) < 0) {
 		if (mkdir(CALLOUT_DIR, mode) < 0) {
-			LOG(1, "cannot create " CALLOUT_DIR);
+			syslog(LOG_ERR, "cannot create " CALLOUT_DIR);
 			return -1;
 		}
-		LOG(3, "created " CALLOUT_DIR);
+		syslog(LOG_DEBUG, "created " CALLOUT_DIR);
 	}
 
 	/*
@@ -704,17 +763,17 @@ prepare_namespace(void)
 		bin = VECTOR_SLOT(binvec, i);
 
 		if ((fd = open(bin, O_RDONLY)) < 0) {
-			LOG(1, "cannot open %s", bin);
+			syslog(LOG_ERR, "cannot open %s", bin);
 			return -1;
 		}
 		if (fstat(fd, &statbuf) < 0) {
-			LOG(1, "cannot stat %s", bin);
+			syslog(LOG_ERR, "cannot stat %s", bin);
 			return -1;
 		}
 		size += statbuf.st_size;
 		close(fd);
 	}
-	LOG(3, "ramfs maxsize is %u", (unsigned int) size);
+	syslog(LOG_INFO, "ramfs maxsize is %u", (unsigned int) size);
 	
 	/*
 	 * mount the ramfs
@@ -724,10 +783,10 @@ prepare_namespace(void)
 		return -1;
 	}
 	if (mount(NULL, CALLOUT_DIR, "ramfs", MS_SYNCHRONOUS, ramfs_args) < 0) {
-		LOG(1, "cannot mount ramfs on " CALLOUT_DIR);
+		syslog(LOG_ERR, "cannot mount ramfs on " CALLOUT_DIR);
 		return -1;
 	}
-	LOG(3, "mount ramfs on " CALLOUT_DIR);
+	syslog(LOG_DEBUG, "mount ramfs on " CALLOUT_DIR);
 
 	/*
 	 * populate the ramfs with callout binaries
@@ -736,10 +795,10 @@ prepare_namespace(void)
 		bin = VECTOR_SLOT(binvec, i);
 		
 		if (copytodir(bin, CALLOUT_DIR) < 0) {
-			LOG(1, "cannot copy %s in ramfs", bin);
+			syslog(LOG_ERR, "cannot copy %s in ramfs", bin);
 			exit_daemon(1);
 		}
-		LOG(3, "cp %s in ramfs", bin);
+		syslog(LOG_DEBUG, "cp %s in ramfs", bin);
 	}
 	strvec_free(binvec);
 
@@ -750,20 +809,20 @@ prepare_namespace(void)
 	 * /tmp  : home of tools temp files
 	 */
 	if (mount(CALLOUT_DIR, "/sbin", NULL, MS_BIND, NULL) < 0) {
-		LOG(1, "cannot bind ramfs on /sbin");
+		syslog(LOG_ERR, "cannot bind ramfs on /sbin");
 		return -1;
 	}
-	LOG(3, "bind ramfs on /sbin");
+	syslog(LOG_DEBUG, "bind ramfs on /sbin");
 	if (mount(CALLOUT_DIR, "/bin", NULL, MS_BIND, NULL) < 0) {
-		LOG(1, "cannot bind ramfs on /bin");
+		syslog(LOG_ERR, "cannot bind ramfs on /bin");
 		return -1;
 	}
-	LOG(3, "bind ramfs on /bin");
+	syslog(LOG_DEBUG, "bind ramfs on /bin");
 	if (mount(CALLOUT_DIR, "/tmp", NULL, MS_BIND, NULL) < 0) {
-		LOG(1, "cannot bind ramfs on /tmp");
+		syslog(LOG_ERR, "cannot bind ramfs on /tmp");
 		return -1;
 	}
-	LOG(3, "bind ramfs on /tmp");
+	syslog(LOG_DEBUG, "bind ramfs on /tmp");
 
 	return 0;
 }
@@ -775,26 +834,26 @@ pidfile (pid_t pid)
 	FILE *file;
 	struct stat *buf;
 
-	buf = MALLOC (sizeof (struct stat));
+	buf = MALLOC(sizeof(struct stat));
 
-	if (!stat (PIDFILE, buf)) {
-		LOG(1, "already running : out");
-		FREE (buf);
-		exit (1);
+	if (!stat(PIDFILE, buf)) {
+		syslog(LOG_ERR, "already running : out");
+		FREE(buf);
+		exit(1);
 	}
 		
-	umask (022);
-	pid = setsid ();
+	umask(022);
+	pid = setsid();
 
 	if (pid < -1) {
-		LOG(1, "setsid() error");
-		exit (1);
+		syslog(LOG_ERR, "setsid() error");
+		exit(1);
 	}
 	
-	file = fopen (PIDFILE, "w");
-	fprintf (file, "%d\n", pid);
-	fclose (file);
-	FREE (buf);
+	file = fopen(PIDFILE, "w");
+	fprintf(file, "%d\n", pid);
+	fclose(file);
+	FREE(buf);
 }
 
 static void *
@@ -819,7 +878,7 @@ signal_set(int signo, void (*func) (int))
 static void
 sighup (int sig)
 {
-	LOG (1, "SIGHUP received from multipath or operator");
+	syslog(LOG_NOTICE, "SIGHUP received from multipath or operator");
 
 	/*
 	 * signal updatepaths() that we come from SIGHUP
@@ -860,7 +919,7 @@ setscheduler (void)
         res = sched_setscheduler (0, SCHED_RR, &sched_param);
 
         if (res == -1)
-                LOG(1, "Could not set SCHED_RR at priority 99");
+                syslog(LOG_WARNING, "Could not set SCHED_RR at priority 99");
 	return;
 }
 
@@ -872,14 +931,16 @@ child (void * param)
 	struct paths *failedpaths;
 
 	openlog("multipathd", 0, LOG_DAEMON);
-	LOG(1, "--------start up--------");
+	setlogmask(LOG_UPTO(LOGLEVEL));
+	syslog(LOG_NOTICE, "--------start up--------");
 
 	pidfile(getpid());
 	signal_init();
 	setscheduler();
 	failedpaths = initpaths();
+	checkint = CHECKINT;
 
-	LOG(2, "read " CONFIGFILE);
+	syslog(LOG_INFO, "read " CONFIGFILE);
 	init_data(CONFIGFILE, init_keywords);
 
 	/*
@@ -905,7 +966,7 @@ child (void * param)
 
 #ifdef CLONE_NEWNS
 	if (prepare_namespace() < 0) {
-		LOG(1, "cannot prepare namespace");
+		syslog(LOG_ERR, "cannot prepare namespace");
 		exit_daemon(1);
 	}
 #endif
